from typing import List

from pydantic import Field

from .base import BaseKeggListModel, BaseKeggWithRefModel


class KeggModuleModel(BaseKeggWithRefModel):
    entry_id: str = Field(regex=r"^M\d{5}$")
    definition: str = None
    orthologies: dict = None
    classes: List[str] = None
    pathways: dict = None
    reactions: dict = None
    compounds: dict = None
    rmodules: dict = None
    comment: str = None


class KeggModuleListModel(BaseKeggListModel):
    pass



import os
from unittest import TestCase

from dabeplech.parsers.kegg import KeggOrthologyParser, KeggOrthologyListParser


class TestKeggOrthologyParser(TestCase):

    def test_parsing(self):
        input_path = os.path.join(os.path.dirname(__file__), 'files/K00135.txt')
        test_file = open(input_path, 'r')
        test_txt = test_file.read()
        test_parser = KeggOrthologyParser(test_txt)
        test_parser.parse()
        tested_entry = test_parser.validated_entry
        # Test some of the attribute and content of the file
        self.assertEqual(tested_entry.entry_id, 'K00135')
        self.assertIn('gabD', tested_entry.names)
        self.assertEqual(tested_entry.definition, 'succinate-semialdehyde dehydrogenase / glutarate-semialdehyde dehydrogenase')  # noqa
        # Test ec numbers list
        self.assertEqual(len(tested_entry.ec_numbers), 3)
        for ec_number in ['1.2.1.16', '1.2.1.79', '1.2.1.20']:
            self.assertIn(ec_number, tested_entry.ec_numbers)
        # Test some keys of dictionnary
        self.assertIn('ko00250', tested_entry.pathways.keys())
        self.assertIn('Alanine, aspartate and glutamate metabolism', tested_entry.pathways.values())
        self.assertIn('M00027', tested_entry.modules.keys())
        self.assertIn('RN', tested_entry.dblinks.dict().keys())
        self.assertIn('LVE', tested_entry.genes.keys())
        # Test references
        self.assertEqual(len(tested_entry.references), 2)
        self.assertEqual(tested_entry.references[0].pubmed_id, 2254272)
        self.assertEqual(tested_entry.references[0].doi, "10.1128/JB.172.12.7035-7042.1990")


class TestKeggOrthologyListParser(TestCase):

    def test_parsing(self):
        input_path = os.path.join(os.path.dirname(__file__), 'files/list_ko.txt')
        test_file = open(input_path, 'r')
        test_txt = test_file.read()
        test_parser = KeggOrthologyListParser(test_txt)
        test_parser.parse()
        tested_model = test_parser.validated_model
        self.assertEqual(len(tested_model.entries), 10)
        # Testing third entry of the list
        tested_entry = tested_model.entries[2]
        self.assertEqual(tested_entry.entry_id, 'K00003')
        self.assertIn('hom', tested_entry.names)
        self.assertEqual(tested_entry.definition, 'homoserine dehydrogenase')
        # Test entry has only expected
        for k in vars(tested_entry).keys():
            self.assertIn(k, ['names', 'entry_id', 'definition', 'ec_numbers'])

    def test_empty_parsing(self):
        input_path = os.path.join(os.path.dirname(__file__), 'files/empty_list.txt')
        test_file = open(input_path, 'r')
        test_txt = test_file.read()
        test_parser = KeggOrthologyListParser(test_txt)
        test_parser.parse()
        tested_model = test_parser.validated_model
        self.assertEqual(len(tested_model.entries), 0)



import logging
from collections import defaultdict

from dabeplech.models.kegg.link import KeggLinksModel
from dabeplech.parsers.base import BaseParser

logging.basicConfig()
logger = logging.getLogger()


class KeggLinkParser(BaseParser):
    """
    Parser for LINK operation from KEGG API.
    """
    model = KeggLinksModel

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.parsed_content['links'] = defaultdict(lambda: list())

    def _parse_line(self, line):
        elements = line.split()
        key_id = elements[0].split(':')[1]
        linked_id = elements[1].split(':')[1]
        self.parsed_content['links'][key_id].append(linked_id)

    def parse(self):
        for line in self.lines:
            self._parse_line(line)



from typing import List

from pydantic import BaseModel


class NCBITaxonomyItemModel(BaseModel):
    tax_id: int
    rank: str
    name: str


class NCBITaxonomyAPIModel(BaseModel):
    current_item: NCBITaxonomyItemModel
    hierarchy: List[NCBITaxonomyItemModel]



from typing import List

from pydantic import BaseModel


class KeggReferenceModel(BaseModel):
    authors: List[str] = None
    title: str
    journal: str
    pubmed_id: int = None
    doi: str = None
    sequence: List[str] = None



import logging

from dabeplech.models.kegg.orthology import KeggOrthologyListModel
from dabeplech.models.kegg.pathway import KeggPathwayListModel
from dabeplech.parsers.base import BaseListParser

logging.basicConfig()
logger = logging.getLogger()


class BaseKeggListParser(BaseListParser):

    def _parse_line(self, line):
        elements = line.split(maxsplit=1)
        entry_id = elements[0].split(':')[1]
        names = elements[1].split(';')[0].split(',')
        names = [name.strip() for name in names]
        return {
            'entry_id': entry_id,
            'names': names,
        }

    def parse(self):
        for line in self.lines:
            if not line.strip():
                continue
            self.parsed_content.append(self._parse_line(line))


class KeggOrthologyListParser(BaseKeggListParser):
    """
    Parser for list of KEGG ko from KEGG API (http://rest.kegg.jp/list/ko).
    """
    model = KeggOrthologyListModel

    def _parse_line(self, line):
        elements = line.split(maxsplit=1)
        entry_id = elements[0].split(':')[1]
        names = elements[1].split(';')[0].split(',')
        names = [name.strip() for name in names]
        if ';' in elements[1]:
            definition = elements[1].split(';')[1].split('[')[0].strip()
        else:
            definition = None
        if '[' in elements[1]:
            ec_numbers = elements[1].split('[')[1].rstrip(']').replace('EC:', '').split()
        else:
            ec_numbers = None
        return {
            'entry_id': entry_id,
            'names': names,
            'definition': definition,
            'ec_numbers': ec_numbers
        }


class KeggPathwayListParser(BaseKeggListParser):
    """
    Parser for list of KEGG pathways (map) from KEGG API (http://rest.kegg.jp/list/pathway).
    """
    model = KeggPathwayListModel


class KeggModuleListParser(BaseKeggListParser):
    """
    Parser for list of KEGG modules (M) from KEGG API (http://rest.kegg.jp/list/module).
    """
    model = KeggPathwayListModel



"""
Script to test the status of different endpoints.

This script is aimed to be run locally to check the availability of different endpoints.
"""
import argparse
import logging
import sys

from kegg_api_status import test_keggapi
from togows_api_status import test_togowsentryapi
from pdbe_api_status import test_pdbeapi
from doi_api_status import test_doiapi

logging.basicConfig()
logger = logging.getLogger()


def parse_arguments():
    parser = argparse.ArgumentParser(description='Test the status of different endpoints supported by dabeplech.')
    parser.add_argument('--verbose', action='store_true')
    parser.add_argument('--debug', action='store_true')

    try:
        return parser.parse_args()
    except SystemExit:
        sys.exit(1)


def run():
    args = parse_arguments()
    if args.verbose:
        logger.setLevel(logging.INFO)
    if args.debug:
        logger.setLevel(logging.DEBUG)
    test_togowsentryapi()
    test_keggapi()
    test_pdbeapi()
    test_doiapi()


if __name__ == "__main__":
    run()



import os
from unittest import TestCase

from dabeplech.parsers.kegg.module import KeggModuleParser
from dabeplech.parsers.kegg.list import KeggModuleListParser


class TestKeggModuleParser(TestCase):

    def test_parsing(self):
        input_path = os.path.join(os.path.dirname(__file__), 'files/M00003.txt')
        test_file = open(input_path, 'r')
        test_txt = test_file.read()
        test_parser = KeggModuleParser(test_txt)
        test_parser.parse()
        tested_entry = test_parser.validated_entry
        # Test some of the attribute and content of the file
        self.assertEqual(tested_entry.entry_id, 'M00003')
        self.assertEqual(tested_entry.definition, "(K01596,K01610) K01689 (K01834,K15633,K15634,K15635) K00927 (K00134,K00150) K01803 ((K01623,K01624,K11645) (K03841,K02446,K11532,K01086,K04041),K01622)")  # noqa
        self.assertIn('Gluconeogenesis', tested_entry.names)
        # Test some keys of dictionnary
        self.assertIn('map00020', tested_entry.pathways.keys())
        self.assertIn('Glycolysis / Gluconeogenesis', tested_entry.pathways.values())
        self.assertIn('R01518', tested_entry.reactions.keys())
        self.assertIn('R00726', tested_entry.reactions.keys())
        self.assertIn('C00111', tested_entry.compounds.keys())

    def test_parsing_empty_lines(self):
        input_path = os.path.join(os.path.dirname(__file__), 'files/M00083.txt')
        test_file = open(input_path, 'r')
        test_txt = test_file.read()
        test_parser = KeggModuleParser(test_txt)
        test_parser.parse()
        tested_entry = test_parser.validated_entry
        # Test some of the attribute and content of the file
        self.assertEqual(tested_entry.entry_id, 'M00083')
        self.assertIn('Fatty acid biosynthesis', tested_entry.names)
        # Test some keys of dictionnary
        self.assertIn('R04959', tested_entry.reactions.keys())


class TestKeggModuleListParser(TestCase):

    def test_parsing(self):
        input_path = os.path.join(os.path.dirname(__file__), 'files/list_module.txt')
        test_file = open(input_path, 'r')
        test_txt = test_file.read()
        test_parser = KeggModuleListParser(test_txt)
        test_parser.parse()
        tested_model = test_parser.validated_model
        self.assertEqual(len(tested_model.entries), 10)
        # Testing third entry of the list
        tested_entry = tested_model.entries[2]
        self.assertEqual(tested_entry.entry_id, 'M00003')
        self.assertIn('Gluconeogenesis', tested_entry.names)
        # Test entry has only expected
        for k in vars(tested_entry).keys():
            self.assertIn(k, ['names', 'entry_id'])



from .version import __version__  # noqa

from .kegg import KEGGAPI  # noqa
from .ncbi_taxonomy import NCBITaxonomyScrapAPI  # noqa
from .togows import TogoWSEntryAPI  # noqa
from .metagenedb import (  # noqa
    MetageneDBTokenAPI,
    MetageneDBCatalogEggNOGAPI,
    MetageneDBCatalogGeneAPI,
    MetageneDBCatalogKeggOrthologyAPI,
    MetageneDBCatalogTaxonomyAPI,
    MetageneDBCatalogFunctionAPI
)
from .pdbe import (  # noqa
    PDBeAPI,
    PDBeUniprotMappingAPI,
    PDBePFAMMappingAPI,
)
from .doi import (  # noqa
    DOIAPI,
)



from dabeplech.base import BaseAPI, GETMixin


class DOIAPI(BaseAPI, GETMixin):
    BASE_URL = "https://dx.doi.org"
    HEADERS = {
        'Content-type': 'application/json',
        'Accept': 'application/vnd.citationstyles.csl+json'
    }



from dabeplech import TogoWSEntryAPI

from utils import _format_print


def _expected_not_working_togowsentry():
    tests = [
        ('kegg-pathway', 'ko05132222'),
        ('kegg-orthology', 'K000011111'),
        ('kegg-orthology', 'no_way')
    ]
    for i in tests:
        db = i[0]
        entry_id = i[1]
        print(f"Test getting {entry_id} from {db} database from {TogoWSEntryAPI.BASE_URL}:")
        try:
            api = TogoWSEntryAPI(db)
            content = api.get(entry_id)  # noqa
            print(_format_print("Status", "OK", "red"))
        except Exception:
            print(_format_print("Status", "ERROR", "green"))
        finally:
            print(f" --> Requested URL: {api.last_url_requested}")


def _expected_working_togowsentry():
    tests = [
        ('kegg-pathway', 'ko05132'),
        ('kegg-orthology', 'K00001'),
    ]
    for i in tests:
        db = i[0]
        entry_id = i[1]
        print(f"Test getting {entry_id} from {db} database from {TogoWSEntryAPI.BASE_URL}:")
        try:
            api = TogoWSEntryAPI(db)
            content = api.get(entry_id)  # noqa
            print(_format_print("Status", "OK", "green"))
            if content:
                print(_format_print("Content", "NOT EMPTY", "green"))
            else:
                print(_format_print("Content", "EMPTY", "red"))
        except Exception:
            print(_format_print("Status", "ERROR", "red"))
        finally:
            print(f" --> Requested URL: {api.last_url_requested}")


def test_togowsentryapi():
    print("---------- Testing TogoWS API endpoints... ----------")
    _expected_working_togowsentry()
    _expected_not_working_togowsentry()



import logging

from dabeplech.models.kegg import KeggPathwayModel
from dabeplech.parsers.kegg.base import BaseKeggParser

logging.basicConfig()
logger = logging.getLogger()


class KeggPathwayKoParser(BaseKeggParser):
    model = KeggPathwayModel


class KeggPathwayMapParser(BaseKeggParser):
    model = KeggPathwayModel

    def _handle_ko_pathway(self, line, **kwargs):
        self.entry.ko_pathway = line.split(maxsplit=1)[-1].strip()


class KeggPathwayParser(KeggPathwayKoParser, KeggPathwayMapParser):
    """
    Parser for KEGG KO `plain text` result from KEGG API.
    """
    model = KeggPathwayModel

    def _handle_pathway_map(self, line, first=False):
        self._simple_handling(line, 'pathway_maps', first=first)

    def _handle_rel_pathway(self, line, first=False):
        self._simple_handling(line, 'related_pathways', first=first)



import logging

from dabeplech.models.kegg.base import BaseKeggModel
from dabeplech.parsers.base import BaseParser
from dabeplech.parsers.kegg.reference import KeggReferenceParser

logging.basicConfig()
logger = logging.getLogger()


class BaseKeggParser(BaseParser):
    """
    Base structure for parsers for KEGG API
    """
    model = BaseKeggModel  # Pydantic model to describe the response

    def __init__(self, content_response: str):
        """
        :param content_response: content response from the API
        """
        self.lines = content_response.rstrip().split('\n')
        self.handler = '_handle_entry'
        self.current_ref = None
        self.skipped_lines = 0

    def _simple_handling(self, line: str, attr_to_set: str, first: bool = False):
        if first:
            elements = line.split(maxsplit=2)
            if len(elements) > 2:
                value = elements[2]
            else:
                value = None
            setattr(self.entry, attr_to_set, {elements[1]: value})
        else:
            elements = line.split(maxsplit=1)
            if len(elements) > 1:
                value = elements[1]
            else:
                value = None
            getattr(self.entry, attr_to_set)[elements[0]] = value

    def _simple_handling_list(self, line: str, attr_to_set: str, first=False):
        if first:
            elements = line.split(maxsplit=2)
            setattr(self.entry, attr_to_set, {elements[1].rstrip(':'): elements[2].split()})
        else:
            elements = line.split(maxsplit=1)
            getattr(self.entry, attr_to_set)[elements[0].rstrip(':')] = elements[1].split()

    def _handle_default(self, line: str, **kwargs):
        logger.info("Skipped line: %s" % line)
        self.skipped_lines += 1

    def _handle_entry(self, line: str, **kwargs):
        entry_id = line.split()[1]
        # Starts a Kegg Orthology object with name that will be updated
        self.entry = self.model(entry_id=entry_id, names=[])

    def _handle_definition(self, line: str, first: bool = False):
        self.entry.definition = line.split(maxsplit=1)[-1]

    def _handle_pathway(self, line: str, first: bool = False):
        self._simple_handling(line, 'pathways', first=first)

    def _handle_brite(self, line: str, **kwargs):
        logger.info("Skipping BRITE part of the response...")
        pass

    def _handle_genes(self, line: str, first: bool = False):
        self._simple_handling_list(line, 'genes', first=first)

    def _handle_name(self, line: str, **kwargs):
        names = line.split(maxsplit=1)[-1].split(",")
        self.entry.names = [name.strip() for name in names]

    def _handle_dblinks(self, line: str, first: bool = False):
        self._simple_handling_list(line, 'dblinks', first=first)

    def _handle_module(self, line: str, first: bool = False):
        self._simple_handling(line, 'modules', first=first)

    def _handle_disease(self, line: str, first: bool = False):
        self._simple_handling(line, 'diseases', first=first)

    def _handle_orthology(self, line: str, first: bool = False):
        self._simple_handling(line, 'orthologies', first=first)

    def _handle_compound(self, line: str, first: bool = False):
        self._simple_handling(line, 'compounds', first=first)

    def _handle_rmodule(self, line: str, first: bool = False):
        self._simple_handling(line, 'rmodules', first=first)

    def _handle_reaction(self, line, first=False):
        if not line.strip():
            return
        if first:
            elements = line.split(maxsplit=2)
            reaction_ids_str = elements[1]
        else:
            elements = line.split(maxsplit=1)
            reaction_ids_str = elements[0]
        reaction_ids = reaction_ids_str.split(',')
        reaction = elements[-1]
        for reaction_id in reaction_ids:
            if getattr(self.entry, 'reactions') is None:
                setattr(self.entry, 'reactions', {reaction_id: reaction})
            else:
                getattr(self.entry, 'reactions')[reaction_id] = reaction

    def _handle_description(self, line: str, **kwargs):
        self.entry.description = line.split(maxsplit=1)[-1]

    def _handle_class(self, line: str, first: bool = False):
        if first:
            self.entry.classes = []
        classes = [element.strip() for element in line.split(maxsplit=1)[-1].split(';')]
        self.entry.classes = self.entry.classes + classes

    def _handle_comment(self, line: str, **kwargs):
        self.entry.comment = line.split(maxsplit=1)[-1].strip()

    def _handle_reference_first(self, line: str):
        if self.current_ref is not None:
            self.entry.references.append(self.current_ref.dict())
        else:  # First time we encounter reference and there will be at least one
            self.entry.references = []
        self.current_ref = KeggReferenceParser()
        self.current_ref.pubmed_id = line

    def _handle_reference_other(self, line: str):
        elements = line.split(maxsplit=1)
        if len(elements) > 1:
            attribute = elements[0].lower()
        else:  # case of handling doi
            attribute = line.strip().split(':')[0].lower()
        setattr(self.current_ref, attribute, line)

    def _handle_reference(self, line: str, first: bool = False):
        if first is True:
            self._handle_reference_first(line)
        else:
            self._handle_reference_other(line)

    def _reattach_last_elements(self):
        """
        This aims to deal with last elements (like in references) that are not appended to their list
        """
        if self.current_ref is not None:
            self.entry.references.append(self.current_ref.dict())

    def parse(self):
        """
        Perform parsing of the text content into the model defined for KEGG orthology.
        """
        for line in self.lines:
            if '///' in line:  # end of lines
                break
            if not line.startswith(' '):  # There is a category to be handled
                category = line.split()[0].lower()
                handler = f"_handle_{category}"
                getattr(self, handler, self._handle_default)(line, first=True)
            else:
                getattr(self, handler, self._handle_default)(line)
        self._reattach_last_elements()

    @property
    def validated_entry(self):
        """
        :return: Validated entry for KEGG orthology.
        :rtype: dict
        """
        # Reparse all the content of entry to make sure it respects the expected structure
        if self.skipped_lines > 0:
            logger.warning("%s lines skipped from parsing for %s", self.skipped_lines, self.entry.entry_id)
        return self.model(**self.entry.dict())



from unittest import TestCase

from dabeplech.parsers.kegg.list import KeggOrthologyListParser, KeggPathwayListParser


class TestKeggOrthologyList(TestCase):

    def test_parse_line(self):
        line = "ko:K00001	E1.1.1.1, adh; alcohol dehydrogenase [EC:1.1.1.1]"
        expected_dict = {
            'entry_id': 'K00001',
            'names': ['E1.1.1.1', 'adh'],
            'definition': 'alcohol dehydrogenase',
            'ec_numbers': ['1.1.1.1']
        }
        parser = KeggOrthologyListParser("test")
        self.assertDictEqual(parser._parse_line(line), expected_dict)

    def test_parse_line_multi_ec_numbers(self):
        line = "ko:K00010	iolG; myo-inositol 2-dehydrogenase / D-chiro-inositol 1-dehydrogenase [EC:1.1.1.18 1.1.1.369]"  # noqa
        expected_dict = {
            'entry_id': 'K00010',
            'names': ['iolG'],
            'definition': 'myo-inositol 2-dehydrogenase / D-chiro-inositol 1-dehydrogenase',
            'ec_numbers': ['1.1.1.18', '1.1.1.369']
        }
        parser = KeggOrthologyListParser("test")
        self.assertDictEqual(parser._parse_line(line), expected_dict)

    def test_parse_line_no_ec_numbers(self):
        line = "ko:K00182	WNT2; wingless-type MMTV integration site family, member 2"
        expected_dict = {
            'entry_id': 'K00182',
            'names': ['WNT2'],
            'definition': 'wingless-type MMTV integration site family, member 2',
            'ec_numbers': None
        }
        parser = KeggOrthologyListParser("test")
        self.assertDictEqual(parser._parse_line(line), expected_dict)

    def test_parse_line_no_definition_no_ec_numbers(self):
        line = "ko:K23479       CCAAT/enhancer binding protein (C/EBP), other"
        expected_dict = {
            'entry_id': 'K23479',
            'names': ['CCAAT/enhancer binding protein (C/EBP)', 'other'],
            'definition': None,
            'ec_numbers': None
        }
        parser = KeggOrthologyListParser("test")
        self.assertDictEqual(parser._parse_line(line), expected_dict)


class TestKeggPathwayList(TestCase):

    def test_parse_line(self):
        line = "path:map00030	Pentose phosphate pathway"
        expected_dict = {
            'entry_id': 'map00030',
            'names': ['Pentose phosphate pathway'],
        }
        parser = KeggPathwayListParser("test")
        self.assertDictEqual(parser._parse_line(line), expected_dict)



import os
from unittest import TestCase

from dabeplech.scrappers.ncbi_taxonomy.taxonomy import NCBITaxonomyScrapper


class TestNCBITaxonomyScrapper(TestCase):

    def setUp(self):
        file_path = os.path.join(os.path.dirname(__file__), 'data/tax_33958.html')
        self.file = open(file_path, "rb")

    def test_result_found(self):
        scrapper = NCBITaxonomyScrapper(self.file)
        self.assertTrue(scrapper.result_found())

    def test_result_found_error_page(self):
        file_path = os.path.join(os.path.dirname(__file__), 'data/error_page.html')
        tax_file = open(file_path, "rb")
        scrapper = NCBITaxonomyScrapper(tax_file)
        self.assertFalse(scrapper.result_found())

    def test_extract_tax_id_from_url(self):
        tested_url = "wwwtax.cgi?mode=Undef&amp;id=131567&amp;lvl=3&amp;keep=1&amp;srchmode=1&amp;unlock"
        expected_id = "131567"
        tested_id = NCBITaxonomyScrapper.extract_tax_id_from_url(tested_url)
        self.assertEqual(tested_id, expected_id)

    def test_retrieve_current_item(self):
        scrapper = NCBITaxonomyScrapper(self.file)
        expected_dict = {
            'rank': 'family',
            'tax_id': '33958',
            'name': 'Lactobacillaceae'
        }
        tested_dict = scrapper.retrieve_current_item()
        self.assertDictEqual(tested_dict, expected_dict)

    def test_retrieve_current_item_no_link(self):
        """
        Some entry has their names without a link, for instance tax_id 339588
        """
        file_path = os.path.join(os.path.dirname(__file__), 'data/tax_339588.html')
        tax_file = open(file_path, "rb")
        scrapper = NCBITaxonomyScrapper(tax_file)
        expected_dict = {
            'rank': 'species',
            'tax_id': '339588',
            'name': 'Peyssonnelia inamoena'
        }
        tested_dict = scrapper.retrieve_current_item()
        self.assertDictEqual(tested_dict, expected_dict)

    def test_retrieve_current_item_variant_1(self):
        """
        Some entry has different display and more information, for instance tax_id 12345
        """
        file_path = os.path.join(os.path.dirname(__file__), 'data/tax_12345.html')
        tax_file = open(file_path, "rb")
        scrapper = NCBITaxonomyScrapper(tax_file)
        expected_dict = {
            'rank': 'species',
            'tax_id': '12345',
            'name': 'Bacillus virus GA1'
        }
        tested_dict = scrapper.retrieve_current_item()
        self.assertDictEqual(tested_dict, expected_dict)

    def test_retrieve_hierarchy(self):
        scrapper = NCBITaxonomyScrapper(self.file)
        expected_list = [
            {'name': 'cellular organisms', 'rank': 'no rank', 'tax_id': '131567'},
            {'name': 'Bacteria', 'rank': 'superkingdom', 'tax_id': '2'},
            {'name': 'Terrabacteria group', 'rank': 'clade', 'tax_id': '1783272'},
            {'name': 'Firmicutes', 'rank': 'phylum', 'tax_id': '1239'},
            {'name': 'Bacilli', 'rank': 'class', 'tax_id': '91061'},
            {'name': 'Lactobacillales', 'rank': 'order', 'tax_id': '186826'}
        ]
        tested_list = scrapper.retrieve_hierarchy()
        self.assertListEqual(tested_list, expected_list)



from bs4 import BeautifulSoup

from dabeplech.models.ncbi_taxonomy.taxonomy import NCBITaxonomyAPIModel


class NCBITaxonomyScrapper:
    """
    Scrapper of html response from NCBI taxonomy info page of a given tax_id
    """
    model = NCBITaxonomyAPIModel

    def __init__(self, html_content: bytes):
        self.soup = BeautifulSoup(html_content, features="html.parser")

    def result_found(self):
        """
        if <h1> is present in the page, it means an error message is returned
        """
        if self.soup.h1 is not None:
            return False
        return True

    @classmethod
    def extract_tax_id_from_url(self, url):
        qparams = url.split('?', maxsplit=1)[1]
        for qparam in qparams.split('&'):
            if 'id=' in qparam:
                return qparam.split('=')[-1]

    def retrieve_current_item(self):
        table_of_interest = self.soup.body.find_all('table')[3]
        name = table_of_interest.find_all('strong')[0].text
        tax_id = table_of_interest.tr.td.text.split("Taxonomy ID:", maxsplit=1)[-1].split()[0]
        rank = table_of_interest.find_all('strong')[-2].text
        current_item = {
            'rank': rank,
            'tax_id': tax_id,
            'name': name
        }
        return current_item

    def retrieve_hierarchy(self):
        hierarchy = []
        for i in self.soup.dd.find_all('a'):
            hierarchy.append({
                'rank': i['title'],
                'tax_id': self.extract_tax_id_from_url(i['href']),
                'name': i.text
            })
        return hierarchy

    def scrap(self):
        """
        Perform scrapping of the content of interest
        """
        self.entry = {}
        self.entry['current_item'] = self.retrieve_current_item()
        self.entry['hierarchy'] = self.retrieve_hierarchy()

    @property
    def validated_entry(self):
        """
        :return: Validated entry in the model described in dabeplech.
        :rtype: dict
        """
        if getattr(self, 'entry', None) is None:
            self.scrap()
        return self.model(**self.entry)



from typing import Union

import requests

from dabeplech.scrappers.ncbi_taxonomy.taxonomy import NCBITaxonomyScrapper


class NCBITaxonomyScrapAPI:
    """
    Scrap NCBI taxonomy pages to return information and mimic API behaviour to retrieve useful
    information such as hierarchy of taxonomy.
    """
    HEADERS = {
        'Content-type': 'text/html',
        'Accept': '*/*'
    }
    SESSION = requests.Session

    def __init__(self):
        self.last_url_requested = None
        self.session = self.SESSION()
        self.session.headers.update(self.HEADERS)

    def get(self, tax_id: int, get_model: bool = True) -> Union[NCBITaxonomyScrapper.model, dict]:
        """
        :param tax_id: NCBI taxonomy ID to retrieve data from
        :param get_model: return pydantic model (return dict if False)
        """
        full_url = f"https://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?id={tax_id}&mode=info"
        response = self.session.get(full_url)
        self.last_url_requested = full_url
        response.raise_for_status()
        scrapper = NCBITaxonomyScrapper(response.content)
        if not scrapper.result_found():
            raise requests.exceptions.HTTPError(f"{tax_id} not found in NCBI taxonomy db.")
        if get_model:
            return scrapper.validated_entry
        return scrapper.validated_entry.dict()
